
from diagrams import Diagram, Cluster

from diagrams.aws.management import Organizations, OrganizationsAccount, OrganizationsOrganizationalUnit

with Diagram("Organizations-State", show=False, direction="TB"):
    ou = OrganizationsOrganizationalUnit("OU")
    oa = OrganizationsAccount("Account")

    with Cluster('Organizations'):

        oo = Organizations('o-gw1gom4vd7\n634252486409\nr-2uoz')

        ou_Bank4usRCI= OrganizationsOrganizationalUnit("ou-2uoz-ysgn3x7o\nBank4usRCI")

        oo>> ou_Bank4usRCI

        ou_ProductsSecurity= OrganizationsOrganizationalUnit("ou-2uoz-y8ozr53i\nProductsSecurity")

        oo>> ou_ProductsSecurity

        ou_DevSecOps= OrganizationsOrganizationalUnit("ou-2uoz-igncnk1m\nDevSecOps")

        oo>> ou_DevSecOps

        ou_Suspended= OrganizationsOrganizationalUnit("ou-2uoz-wn4jcrya\nSuspended")

        oo>> ou_Suspended

        ou_Infrastructure= OrganizationsOrganizationalUnit("ou-2uoz-t49i8js2\nInfrastructure")

        oo>> ou_Infrastructure

        ou_WorkLoadsRCI= OrganizationsOrganizationalUnit("ou-2uoz-qsot2f01\nWorkLoadsRCI")

        ou_Bank4usRCI>> ou_WorkLoadsRCI

        ou_InfrastructureRCI= OrganizationsOrganizationalUnit("ou-2uoz-wer9cp68\nInfrastructureRCI")

        ou_Bank4usRCI>> ou_InfrastructureRCI

        ou_PlayGroundRCI= OrganizationsOrganizationalUnit("ou-2uoz-y42c2jx1\nPlayGroundRCI")

        ou_Bank4usRCI>> ou_PlayGroundRCI

        ou_WorkLoadsRCI>> OrganizationsAccount("925371814738\naws-qa-bank4us-p\nroduct-sophos")

        oo >> OrganizationsAccount("634252486409\ncloud-saas@sophossolutions.com")

        ou_InfrastructureRCI>> OrganizationsAccount("318253146201\naws-shared-bank4\nus-product-sophos")

        ou_PlayGroundRCI>> OrganizationsAccount("450073187267\naws-test-product\n-sophos")

        ou_ProductsSecurity>> OrganizationsAccount("675404175100\naws-security-pro\nduct-sophos")

        ou_ProductsSecurity>> OrganizationsAccount("714892466397\naws-logsbackups-\nproduct-sophos")

        ou_DevSecOps>> OrganizationsAccount("124962754109\naws-devsecops-pr\noduct-sophos")

        ou_WorkLoadsRCI>> OrganizationsAccount("151377982441\naws-dev-bank4us-\nproduct-sophos")

        ou_InfrastructureRCI>> OrganizationsAccount("753541840841\naws-networking-b\nank4us-products-sophos")

        ou_Infrastructure>> OrganizationsAccount("818468602340\naws-networking-p\nroduct-sophos")

        ou_WorkLoadsRCI>> OrganizationsAccount("872865219972\naws-prod-bank4us\n-product-sophos")

        ou_Infrastructure>> OrganizationsAccount("570700223503\naws-shared-produ\nct-sophos")



import boto3
import logging

def list_instances(client=boto3.client('sso-admin', region_name="us-east-2")):
    response = client.list_instances(

    )

    return response["Instances"]


def list_account_assignments(instance_arn, account_id,
                             permission_set_arn,
                             client=boto3.client('sso-admin', region_name="us-east-2"), ):
    response = client.list_account_assignments(
        InstanceArn=instance_arn,
        AccountId=account_id,
        PermissionSetArn=permission_set_arn,

    )

    return response["AccountAssignments"]


def list_permissions_set(instance_arn, client=boto3.client('sso-admin', region_name="us-east-2"), ):
    response = client.list_permission_sets(
        InstanceArn=instance_arn,

    )
    logging.debug(response)
    return response["PermissionSets"]


def list_permission_provisioned(account_id, instance_arn, client=boto3.client('sso-admin', region_name="us-east-2"), ):
    response = client.list_permission_sets_provisioned_to_account(
        InstanceArn=instance_arn,
        AccountId=account_id,

    )
    logging.debug(response)
    return response["PermissionSets"]


def extends_permissions_set(permissions_sets, store_arn, client_sso=boto3.client('sso-admin', region_name="us-east-2")):
    l_permissions_set_arn_name = {}
    for p in permissions_sets:
        response = client_sso.describe_permission_set(
            InstanceArn=store_arn,
            PermissionSetArn=p
        )

        l_permissions_set_arn_name[p] = response["PermissionSet"]["Name"]

        logging.debug(response["PermissionSet"]["Name"])
    return l_permissions_set_arn_name



import boto3
from .describe_sso import list_account_assignments
from colorama import Fore
import logging


def list_groups(identity_store_id, client=boto3.client('identitystore', region_name="us-east-2"), ):
    response = client.list_groups(
        IdentityStoreId=identity_store_id,

    )

    return response["Groups"]


def list_users(identity_store_id, client=boto3.client('identitystore', region_name="us-east-2"), ):
    response = client.list_users(
        IdentityStoreId=identity_store_id,

    )

    return response["Users"]


def get_members(identity_store_id, groups, client=boto3.client('identitystore', region_name="us-east-2")):
    group_members = []
    for g in groups:
        response = client.list_group_memberships(
            IdentityStoreId=identity_store_id,
            GroupId=g["GroupId"],

        )
        group_members.append(
            {"group_id": g["GroupId"],
             "group_name": g["DisplayName"],
             "members": response["GroupMemberships"]
             }
        )

    return group_members


def complete_group_members(group_members, users_list):
    for m in group_members:
        for u in m["members"]:
            for a in users_list:
                if u["MemberId"]["UserId"] == a["UserId"]:
                    u["MemberId"]["UserName"] = a["UserName"]

    return group_members


def extend_account_assignments(accounts_list, permissions_sets, store_arn,
                               client_sso=boto3.client('identitystore', region_name="us-east-2")):
    account_assignments = []
    for p in permissions_sets:

        for ac in accounts_list:
            assign = list_account_assignments(instance_arn=store_arn, account_id=ac["Id"], client=client_sso,
                                              permission_set_arn=p)
            logging.debug(f"AccountAssignments  {assign}")
            for a in assign:
                account_assignments.append(a)
    return account_assignments


def add_users_and_groups_assign(account_assignments_list, user_and_group_list, user_list,
                                list_permissions_set_arn_name):
    for a in account_assignments_list:
        for g in user_and_group_list:
            if len(a) > 0 and a['PrincipalType'] == 'GROUP' and g["group_id"] == a['PrincipalId']:
                print( Fore.YELLOW +
                    f"Account {a['AccountId']} assign to {a['PrincipalType']} {g['group_name']} with permission set {list_permissions_set_arn_name[a['PermissionSetArn']]} or {a['PermissionSetArn']}" + Fore.RESET)

                a["GroupName"] = g['group_name']
                a["PermissionSetName"] = list_permissions_set_arn_name[a['PermissionSetArn']]
        for u in user_list:
            if len(a) > 0 and a['PrincipalType'] == 'USER' and u["UserId"] == a['PrincipalId']:
                print(Fore.YELLOW +
                    f"Account {a['AccountId']} assign to {a['PrincipalType']} {u['UserName']} with permission set {a['PermissionSetArn']} or {list_permissions_set_arn_name[a['PermissionSetArn']]}"+ Fore.RESET)
                a["UserName"] = u['UserName']
                a["PermissionSetName"] = list_permissions_set_arn_name[a['PermissionSetArn']]
    logging.debug(f"Account Assignments --> {account_assignments_list}")
    return account_assignments_list


def order_accounts_assignments_list(accounts_dict, account_assignments):
    final_account_assignments = {}
    for ac in accounts_dict:
        final_account_assignments[ac["Name"]] = []
        for a in account_assignments:

            if ac["Id"] == a["AccountId"]:
                final_account_assignments[ac["Name"]].append(a)

        logging.debug(f"Final Account Assignments: {final_account_assignments}")
    return final_account_assignments



import logging
import re

def split_recursive(b_string):
    unit=16
    c_string= b_string[:unit]
    if len(b_string)>unit:
        a=0
        while a< len(b_string) :
            c_string += b_string[unit :] + "\\n"
            a+=unit
    return c_string


def format_name_string(a_string, action=None):
    """

    :param a_string:
    :param action:
    :return: sring --terragrunt-json-out
    """
    if action == 'split':
        if len(a_string) > 17:
            #a_string = a_string[:16] + "\\n" + a_string[16:]
            a_string= split_recursive(a_string)

    elif action == 'format':

        a_string = re.sub('[-!?@.+]', '', a_string)
        a_string = a_string.replace(" ", '')
    return a_string


st= "151377982441-aws-dev-bank4us-product-sophos"

print(format_name_string(st, "split"))